@@ -84,6 +84,7 @@ import qualified Prelude
 --
 -- [/composition/]
 --   @'mapMaybe' f . 'mapMaybe' g ≡ 'mapMaybe' (f <=< g)@
+--
 class Functor f => Filterable f where
   -- | Like 'Maybe.mapMaybe'.
   mapMaybe :: (a -> Maybe b) -> f a -> f b
@@ -100,6 +101,14 @@ class Functor f => Filterable f where
   filter f = mapMaybe $ \a -> if f a then Just a else Nothing
   {-# INLINE filter #-}
 
+  -- | Empty a filterable.
+  --
+  -- @'drain' ≡ 'mapMaybe' (const Nothing)@
+  --
+  drain :: f a -> f b
+  drain = mapMaybe (const Nothing)
+  {-# INLINE drain #-}
+
   {-# MINIMAL mapMaybe | catMaybes #-}
 
 -- | An enhancement of 'Traversable' with 'Filterable'
@@ -169,6 +178,7 @@ class (T.Traversable t, Filterable t) => Witherable t where
 
 instance Filterable Maybe where
   mapMaybe f = (>>= f)
+  drain _ = Nothing
   {-# INLINE mapMaybe #-}
 
 instance Witherable Maybe where
@@ -180,6 +190,7 @@ instance Witherable Maybe where
 
 instance Filterable Option where
   mapMaybe f = (>>= Option . f)
+  drain _ = Option Nothing
   {-# INLINE mapMaybe #-}
 
 instance Witherable Option where
@@ -197,6 +208,9 @@ instance Monoid e => Filterable (Either e) where
   mapMaybe f (Right a) = maybe (Left mempty) Right $ f a
   {-# INLINABLE mapMaybe #-}
 
+  drain (Left e)  = Left e
+  drain (Right _) = Left mempty
+
 instance Monoid e => Witherable (Either e) where
   wither _ (Left e) = pure (Left e)
   wither f (Right a) = fmap (maybe (Left mempty) Right) (f a)
@@ -206,11 +220,13 @@ instance Filterable [] where
   mapMaybe = Maybe.mapMaybe
   catMaybes = Maybe.catMaybes
   filter = Prelude.filter
+  drain _ = []
 
 instance Filterable ZipList where
   mapMaybe f = ZipList . Maybe.mapMaybe f . getZipList
   catMaybes = ZipList . Maybe.catMaybes . getZipList
   filter f = ZipList . Prelude.filter f . getZipList
+  drain _ = ZipList []
 
 -- | Methods are good consumers for fusion.
 instance Witherable [] where
@@ -236,12 +252,14 @@ instance Witherable ZipList where
 instance Filterable IM.IntMap where
   mapMaybe = IM.mapMaybe
   filter = IM.filter
+  drain _ = IM.empty
 
 instance Witherable IM.IntMap where
 
 instance Filterable (M.Map k) where
   mapMaybe = M.mapMaybe
   filter = M.filter
+  drain _ = M.empty
 
 instance Witherable (M.Map k) where
 #if MIN_VERSION_containers(0,5,8)
@@ -251,6 +269,7 @@ instance Witherable (M.Map k) where
 instance (Eq k, Hashable k) => Filterable (HM.HashMap k) where
   mapMaybe = HM.mapMaybe
   filter = HM.filter
+  drain _ = HM.empty
 
 instance (Eq k, Hashable k) => Witherable (HM.HashMap k) where
 
@@ -271,6 +290,7 @@ instance Witherable (Const r) where
 instance Filterable V.Vector where
   filter   = V.filter
   mapMaybe = V.mapMaybe
+  drain _  = V.empty
 
 instance Witherable V.Vector where
   wither f = fmap V.fromList . wither f . V.toList
@@ -283,6 +303,7 @@ instance Filterable S.Seq where
   mapMaybe f = S.fromList . mapMaybe f . F.toList
   {-# INLINABLE mapMaybe #-}
   filter = S.filter
+  drain _ = S.empty
 
 instance Witherable S.Seq where
   wither f = fmap S.fromList . wither f . F.toList
@@ -313,6 +334,7 @@ instance (Functor f, Filterable g) => Filterable (Compose f g) where
   mapMaybe f = Compose . fmap (mapMaybe f) . getCompose
   filter p = Compose . fmap (filter p) . getCompose
   catMaybes = Compose . fmap catMaybes . getCompose
+  drain = Compose . fmap drain . getCompose
 
 instance (T.Traversable f, Witherable g) => Witherable (Compose f g) where
   wither f = fmap Compose . T.traverse (wither f) . getCompose
@@ -323,6 +345,7 @@ instance (Filterable f, Filterable g) => Filterable (P.Product f g) where
   mapMaybe f (P.Pair x y) = P.Pair (mapMaybe f x) (mapMaybe f y)
   filter p (P.Pair x y) = P.Pair (filter p x) (filter p y)
   catMaybes (P.Pair x y) = P.Pair (catMaybes x) (catMaybes y)
+  drain (P.Pair x y) = P.Pair (drain x) (drain y)
 
 instance (Witherable f, Witherable g) => Witherable (P.Product f g) where
   wither f (P.Pair x y) = liftA2 P.Pair (wither f x) (wither f y)
@@ -339,6 +362,9 @@ instance (Filterable f, Filterable g) => Filterable (Sum.Sum f g) where
   filter p (Sum.InL x) = Sum.InL (filter p x)
   filter p (Sum.InR y) = Sum.InR (filter p y)
 
+  drain (Sum.InL x) = Sum.InL (drain x)
+  drain (Sum.InR y) = Sum.InR (drain y)
+
 instance (Witherable f, Witherable g) => Witherable (Sum.Sum f g) where
   wither f (Sum.InL x) = Sum.InL <$> wither f x
   wither f (Sum.InR y) = Sum.InR <$> wither f y
@@ -470,13 +496,13 @@ class (FunctorWithIndex i t, Filterable t) => FilterableWithIndex i t | t -> i w
   imapMaybe f = catMaybes . imap f
   {-# INLINE imapMaybe #-}
 
-  -- | @'ifilter' f . 'ifilter' g ≡ ifilter (\i -> 'liftA2' ('&&') (f i) (g i))@
+  -- | @'filter' f . 'ifilter' g ≡ ifilter (\i x -> f x '&&' g i x)@
   ifilter :: (i -> a -> Bool) -> t a -> t a
   ifilter f = imapMaybe $ \i a -> if f i a then Just a else Nothing
   {-# INLINE ifilter #-}
 
 -- | Indexed variant of 'Witherable'.
-class (TraversableWithIndex i t, Witherable t) => WitherableWithIndex i t | t -> i where
+class (TraversableWithIndex i t, FilterableWithIndex i t, Witherable t) => WitherableWithIndex i t | t -> i where
   -- | Effectful 'imapMaybe'.
   --
   -- @'iwither' (\ i -> 'pure' . f i) ≡ 'pure' . 'imapMaybe' f@
@@ -669,7 +695,7 @@ hashNub :: (Witherable t, Eq a, Hashable a) => t a -> t a
 hashNub = hashNubOn id
 {-# INLINE hashNub #-}
 
--- | The 'hashNubOn' function behaves just like 'ordNub',
+-- | The 'hashNubOn' function behaves just like 'hashNub',
 --   except it uses a another type to determine equivalence classes.
 --
 -- >>> hashNubOn fst [(True, 'x'), (False, 'y'), (True, 'z')]
